@@ -13,6 +13,14 @@
 //!
 //!Bug in date - Jun 5, 1825 doesn't work properly.
 //!
+//!Features
+//!
+//!minimal exposes a minimal interface.
+//!
+//!builtin exposes an interface that allows extra SQL builtin functions to be defined.
+//!
+//!internal (the default) exposes a maximal interface.
+//!
 //! Database with SQL-like language.
 //! Example program:
 //! ```
@@ -81,105 +89,9 @@ use std::{
 #[cfg(feature = "internal")]
 #[macro_use]
 pub mod util;
-#[cfg(feature = "internal")]
-/// Compilation of builtin functions.
-pub mod builtin;
-#[cfg(feature = "internal")]
-/// Storage of variable length values : ByteStorage.
-pub mod bytes;
-#[cfg(feature = "internal")]
-/// Stash to retain page values at different times.
-pub mod stash;
-#[cfg(feature = "internal")]
-/// Structs that implement CExp trait.
-pub mod cexp;
-#[cfg(feature = "internal")]
-/// CompactFile : storage of logical pages in smaller regions of backing storage.
-pub mod compact;
-#[cfg(feature = "internal")]
-/// Functions to compile parsed expressions, checking types.
-pub mod compile;
-#[cfg(feature = "internal")]
-/// Instruction execution.
-pub mod exec;
-#[cfg(feature = "internal")]
-/// Expression types, result of parsing.
-pub mod expr;
-#[cfg(feature = "internal")]
-/// Page for SortedFile.
-pub mod page;
-#[cfg(feature = "internal")]
-/// Parser.
-pub mod parse;
-/// Paged data storage.
-pub mod pstore;
-#[cfg(feature = "internal")]
-/// Instruction and other run time types.
-pub mod run;
-#[cfg(feature = "internal")]
-/// Sorted Record storage.
-pub mod sortedfile;
-#[cfg(feature = "internal")]
-/// System table functions.
-pub mod sys;
-#[cfg(feature = "internal")]
-/// Table, ColInfo, Row and other Table types.
-pub mod table;
-#[cfg(feature = "internal")]
-/// Run-time Value.
-pub mod value;
-
-#[cfg(feature = "minimal")]
+#[cfg(not(feature = "internal"))]
 #[macro_use]
 mod util;
-#[cfg(feature = "minimal")]
-/// Compilation of builtin functions.
-mod builtin;
-#[cfg(feature = "minimal")]
-/// Storage of variable length values : ByteStorage.
-mod bytes;
-#[cfg(feature = "minimal")]
-/// Stash to retain page values at different times.
-mod stash;
-#[cfg(feature = "minimal")]
-/// Structs that implement CExp trait.
-mod cexp;
-#[cfg(feature = "minimal")]
-/// CompactFile : storage of logical pages in smaller regions of backing storage.
-mod compact;
-#[cfg(feature = "minimal")]
-/// Functions to compile parsed expressions, checking types.
-mod compile;
-#[cfg(feature = "minimal")]
-/// Instruction execution.
-mod exec;
-#[cfg(feature = "minimal")]
-/// Expression types, result of parsing.
-mod expr;
-
-#[cfg(feature = "minimal")]
-/// Page for SortedFile.
-mod page;
-#[cfg(feature = "minimal")]
-/// Parser.
-mod parse;
-
-#[cfg(feature = "minimal")]
-/// Instruction and other run time types.
-mod run;
-#[cfg(feature = "minimal")]
-/// Sorted Record storage.
-mod sortedfile;
-
-#[cfg(feature = "minimal")]
-/// System table functions.
-mod sys;
-#[cfg(feature = "minimal")]
-/// Table, ColInfo, Row and other Table types.
-mod table;
-#[cfg(feature = "minimal")]
-/// Run-time Value.
-mod value;
 
 // Modules that are always public.
 
@@ -189,12 +101,107 @@ pub mod genquery;
 /// Initial SQL
 pub mod init;
 
-/// Backing storage for CompactFile.
+/// Backing storage for database.
 pub mod stg;
 
 /// WebQuery struct for making a http web server.
 pub mod web;
 
+/// Page storage.
+pub mod pstore;
+
+// Conditional modules.
+
+#[cfg(feature = "internal")]
+/// Compilation of builtin functions.
+pub mod builtin;
+#[cfg(not(feature = "internal"))]
+mod builtin;
+
+#[cfg(feature = "internal")]
+/// Storage of variable length values : ByteStorage.
+pub mod bytes;
+#[cfg(not(feature = "internal"))]
+mod bytes;
+
+#[cfg(feature = "internal")]
+/// Structs that implement CExp trait.
+pub mod cexp;
+#[cfg(not(feature = "internal"))]
+mod cexp;
+
+#[cfg(feature = "internal")]
+/// CompactFile : storage of logical pages in smaller regions of backing storage.
+pub mod compact;
+#[cfg(not(feature = "internal"))]
+mod compact;
+
+#[cfg(not(feature = "minimal"))]
+/// Functions to compile parsed expressions, checking types.
+pub mod compile;
+#[cfg(feature = "minimal")]
+mod compile;
+
+#[cfg(feature = "internal")]
+/// Instruction execution.
+pub mod exec;
+#[cfg(not(feature = "internal"))]
+mod exec;
+
+#[cfg(not(feature = "minimal"))]
+/// Expression types, result of parsing.
+pub mod expr;
+#[cfg(feature = "minimal")]
+mod expr;
+
+#[cfg(feature = "internal")]
+/// Page for SortedFile.
+pub mod page;
+#[cfg(not(feature = "internal"))]
+mod page;
+
+#[cfg(feature = "internal")]
+/// Parser.
+pub mod parse;
+#[cfg(not(feature = "internal"))]
+mod parse;
+
+#[cfg(feature = "internal")]
+/// Instruction and other run time types.
+pub mod run;
+#[cfg(not(feature = "internal"))]
+mod run;
+
+#[cfg(feature = "internal")]
+/// Sorted Record storage.
+pub mod sortedfile;
+#[cfg(not(feature = "internal"))]
+mod sortedfile;
+
+#[cfg(feature = "internal")]
+/// Stash to retain page values at different times.
+pub mod stash;
+#[cfg(not(feature = "internal"))]
+mod stash;
+
+#[cfg(feature = "internal")]
+/// System table functions.
+pub mod sys;
+#[cfg(not(feature = "internal"))]
+mod sys;
+
+#[cfg(feature = "internal")]
+/// Table, ColInfo, Row and other Table types.
+pub mod table;
+#[cfg(not(feature = "internal"))]
+mod table;
+
+#[cfg(feature = "internal")]
+/// Run-time Value.
+pub mod value;
+#[cfg(not(feature = "internal"))]
+mod value;
+
 // End of modules.
 
 /// ```Arc<Vec<u8>>```
@@ -321,7 +328,7 @@ GO
         db
     }
 
-    #[cfg(feature = "internal")]
+    #[cfg(not(feature = "minimal"))]
     /// Register a builtin function.
     pub fn register(self: &DB, name: &str, typ: DataKind, cf: CompileFunc) {
         self.builtins
@@ -384,6 +391,7 @@ GO
             None
         }
     }
+
     /// Save updated tables to underlying file ( or rollback if there was an error ).
     pub fn save(self: &DB) {
         let op = if self.err.get() {
@@ -417,6 +425,7 @@ GO
         self.file.save(op);
         // self.dump_tables();
     }
+
     /// Get the named table.
     pub fn get_table(self: &DB, name: &ObjRef) -> Option<TablePtr> {
         if let Some(t) = self.tables.borrow().get(name) {
@@ -424,6 +433,7 @@ GO
         }
         sys::get_table(self, name)
     }
+
     /// Get the named function.
     pub fn get_function(self: &DB, name: &ObjRef) -> Option<FunctionPtr> {
         if let Some(f) = self.functions.borrow().get(name) {
@@ -431,11 +441,13 @@ GO
         }
         sys::get_function(self, name)
     }
+
     /// Insert the table into the map of tables.
     pub fn publish_table(&self, table: TablePtr) {
         let name = table.info.name.clone();
         self.tables.borrow_mut().insert(name, table);
     }
+
     /// Get code for value.
     pub fn encode(self: &DB, val: &Value) -> u64 {
         let bytes = match val {
@@ -450,19 +462,23 @@ GO
         }
         self.bs.encode(self, &bytes[7..])
     }
+
     /// Decode u64 to bytes.
     pub fn decode(self: &DB, code: u64) -> Vec<u8> {
         self.bs.decode(self, code)
     }
+
     /// Delete encoding.
     pub fn delcode(self: &DB, code: u64) {
         self.bs.delcode(self, code);
     }
+
     /// Allocate a page of underlying file storage.
     pub fn alloc_page(self: &DB) -> u64 {
         self.file.alloc_page()
     }
-    /// Free a pagee of underyling file storage.
+
+    /// Free a page of underyling file storage.
     pub fn free_page(self: &DB, lpnum: u64) {
         self.file.free_page(lpnum);
     }
