@@ -70,22 +70,22 @@ impl SharedPagedData {
     }
 
     fn get_page(&self, lpnum: u64, time: u64, writer: bool) -> Data {
-        let p = {
-            let x = self.x.read().unwrap();
-            if !writer {
-                if let Some(p) = x.stash.get(lpnum, time) {
-                    return p.clone();
-                }
-            }
-            if let Some(p) = x.cache.get(&lpnum) {
+        let mut x = self.x.write().unwrap();
+
+        if !writer {
+            if let Some(p) = x.stash.get(lpnum, time) {
                 return p.clone();
             }
-            let n = x.file.page_size(lpnum);
-            let mut v = vec![0; n];
-            x.file.get_page(lpnum, &mut v);
-            Arc::new(v)
-        };
-        let mut x = self.x.write().unwrap();
+        }
+        if let Some(p) = x.cache.get(&lpnum) {
+            return p.clone();
+        }
+
+        // Page not cached, read from file.
+        let n = x.file.page_size(lpnum);
+        let mut v = vec![0; n];
+        x.file.get_page(lpnum, &mut v);
+        let p = Arc::new(v);
         x.cache.insert(lpnum, p.clone());
         p
     }
